from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session
from requests.compat import urljoin

from kosapi import settings
from .resource import Resource


class KosapiClient:
    def __init__(self, client_id=None, client_secret=None):
        def token_saver(token):
            self._token = token

        if client_id is None:
            client_id = settings.client_settings['client_id']
        if client_secret is None:
            client_secret = settings.client_settings['client_secret']

        self._token = None
        self._auth_dict = {'client_id': client_id,
                           'client_secret': client_secret}

        self.session = OAuth2Session(
            client=BackendApplicationClient(client_id=client_id),
            auto_refresh_kwargs=self._auth_dict,
            auto_refresh_url=settings.API_URLS['refresh_url'],
            token_updater=token_saver)

        resp = self.session.fetch_token(token_url=settings.API_URLS['token_url'],
                                        client_id=client_id,
                                        client_secret=client_secret)
        token_saver(resp['access_token'])

    def resource(self, url, **kwargs):
        for entry in Resource(self, url, **kwargs):
            yield entry

    def get(self, url, **params):
        resp = self.session.get(url, params=params)
        return resp

    def get_relative(self, url, **params):
        url = self._build_url(url)
        resp = self.session.get(url, params=params)
        return resp

    def _build_url(self, url):
        return urljoin(settings.API_URLS['base_url'], url)
